﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using devP2P;


namespace SimpleP2P
{
    public partial class Form1 : Form
    {

        // globals - to keep garbage collector from releasing them
        P2P.ForwardOpenEvent MyForwardOpenEvent;
        P2P.LinkDoneEvent MyLinkDoneEvent;
        P2P.ForwardCloseEvent MyForwardCloseEvent;
        P2P.StoppedEvent MyStoppedEvent;
        P2P.TextReceivedEvent MyTextReceivedEvent;
        P2P.SearchDoneEvent MySearchDoneEvent;
        P2P.StateChangeEvent MyStateChangedEvent;
        P2P.UserConnectedEvent MyUserConnectedEvent;
        P2P.UserDisconnectedEvent MyUserDisconnectedEvent;
        P2P.PingEvent MyPingEvent;

        P2P.FileDoneEvent MyFileDoneEvent;
        P2P.FileProgressEvent MyFireProgressEvent;
        P2P.FileReceiveEvent MyFileReceiveEvent;

        public Form1()
        {
            InitializeComponent();

            MyForwardOpenEvent = new P2P.ForwardOpenEvent(P2P_ForwardOpenEventImpl);
            MyLinkDoneEvent = new P2P.LinkDoneEvent(P2P_LinkDoneEventImpl);
            MyForwardCloseEvent = new P2P.ForwardCloseEvent(P2P_ForwardCloseEventImpl);
            MyStoppedEvent = new P2P.StoppedEvent(P2P_StoppedEventImpl);
            MyTextReceivedEvent = new P2P.TextReceivedEvent(P2P_TextReceivedEventImpl);
            MySearchDoneEvent = new P2P.SearchDoneEvent(P2P_SearchDoneEventImpl);
            MyStateChangedEvent = new P2P.StateChangeEvent(P2P_StateChangeEventImpl);
            MyUserConnectedEvent = new P2P.UserConnectedEvent(P2P_UserConnectedEventImpl);
            MyUserDisconnectedEvent = new P2P.UserDisconnectedEvent(P2P_UserDisconnectedEventImpl);
            MyPingEvent = new P2P.PingEvent(P2P_PingEventImpl);

            MyFileDoneEvent = new P2P.FileDoneEvent(p2p_FileDoneEventImpl);
            MyFireProgressEvent = new P2P.FileProgressEvent(p2p_FileProgressEventImpl);
            MyFileReceiveEvent = new P2P.FileReceiveEvent(p2p_FileReceiveEventImpl);

        }
        public IntPtr p2phandle;

        int TickCounter;
        private void Form1_FormClosed(object sender, System.Windows.Forms.FormClosedEventArgs e)
        {
            if (p2phandle != IntPtr.Zero)
                P2P.Destroy(p2phandle);

               //let's enumerate all forms
            if (Application.OpenForms.Count == 0)
            {
                Application.Exit();
            }
        }

        private void btnSearch_Click(System.Object eventSender, System.EventArgs eventArgs)
        {
            TickCounter = 0;

            P2P.SetMediatorAddress(p2phandle, "mediator.weonlydo.com");
            P2P.SetMediatorPort(p2phandle, 8002);
            P2P.SetPeerName(p2phandle, txtPeerName.Text);
            P2P.Search(p2phandle, null);

            AddToList("Searching for " + txtPeerName.Text);
        }

        private void btnConnect_Click(System.Object eventSender, System.EventArgs eventArgs)
        {
            P2P.Link(p2phandle, Text6.Text);
        }

        private void btnStop_Click(System.Object eventSender, System.EventArgs eventArgs)
        {
            P2P.Stop(p2phandle);
        }

        private void btnSend_Click(System.Object eventSender, System.EventArgs eventArgs)
        {
            P2P.SendText(p2phandle, 1, Text5.Text, true);
        }

        private void btnAdd_Click(System.Object eventSender, System.EventArgs eventArgs)
        {
            int e = 0;
            StringBuilder buffer = new StringBuilder(512);

            IntPtr forwhandle = default(IntPtr);
            forwhandle = P2P.GetForward(p2phandle, 1);

            P2P.Forward_SetForwardType(forwhandle, (int)P2P.ForwardTypes.TCPLocalListen);
            P2P.Forward_SetLocalAddress(forwhandle, Text7.Text);
            P2P.Forward_SetLocalPort(forwhandle, Convert.ToInt32(Text8.Text));
            P2P.Forward_SetRemoteAddress(forwhandle, Text9.Text);
            P2P.Forward_SetRemotePort(forwhandle, Convert.ToInt32(txtMyName0.Text));

            e = P2P.StartForward(p2phandle, 1);
            if (e != 0)
            {
                P2P.ErrorText(p2phandle, e, buffer);
                MessageBox.Show(buffer.ToString());
            }
        }

        private void Form1_Load(System.Object sender, System.EventArgs e)
        {
            try
            {
                p2phandle = P2P.Create();
            }
            catch (System.Exception)
            {
                MessageBox.Show("Please check your P2P.dll path in P2P.cs source of this project!", "Failed to load P2P.dll");
                Application.Exit();
                return;            	
            }

            P2P.RegisterForwardOpenEvent(p2phandle, MyForwardOpenEvent);
            P2P.RegisterForwardCloseEvent(p2phandle, MyForwardCloseEvent);
            P2P.RegisterLinkDoneEvent(p2phandle, MyLinkDoneEvent);
            P2P.RegisterStoppedEvent(p2phandle, MyStoppedEvent);
            P2P.RegisterTextReceivedEvent(p2phandle, MyTextReceivedEvent);
            P2P.RegisterSearchDoneEvent(p2phandle, MySearchDoneEvent);
            P2P.RegisterStateChangeEvent(p2phandle, MyStateChangedEvent);
            P2P.RegisterUserConnectedEvent(p2phandle, MyUserConnectedEvent);
            P2P.RegisterUserDisconnectedEvent(p2phandle, MyUserDisconnectedEvent);
            P2P.RegisterPingEvent(p2phandle, MyPingEvent);

            P2P.RegisterFileDoneEvent(p2phandle, MyFileDoneEvent);
            P2P.RegisterFileProgressEvent(p2phandle, MyFireProgressEvent);
            P2P.RegisterFileReceiveEvent(p2phandle, MyFileReceiveEvent);

            EnableButtons();

            StringBuilder buffer = new StringBuilder(512);
            P2P.GetVersion(p2phandle, buffer);
            AddToList("devP2P VERSION " + buffer.ToString());
        }

        private delegate void EnableButtonsDelegate();

        private void EnableButtons()
        {
            int state = 0;

            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new EnableButtonsDelegate(EnableButtons));

            }
            else
            {
                Timer1.Enabled = false;
                TickCounter = 0;
                btnConnect.BackColor = Color.Transparent;
                btnSearch.BackColor = Color.Transparent;
                state = P2P.GetState(p2phandle);

                if (state > 0)
                {
                    // started
                    txtMyName.Enabled = false;
                    txtPassword.Enabled = false;
                    txtPeerName.Enabled = false;

                    btnStart.Enabled = false;
                    btnStop.Enabled = true;

                    if (state > 4)
                    {
                        // connected
                        btnConnect.Enabled = false;
                        btnSend.Enabled = true;
                        btnSendFile.Enabled = true;
                        btnAdd.Enabled = true;
                        btnSearch.Enabled = false;
                    }
                    else
                    {
                        if (!string.IsNullOrEmpty(Text6.Text))
                        {
                            btnConnect.Enabled = true;
                        }
                        else
                        {
                            btnConnect.Enabled = false;
                        }
                        btnSend.Enabled = false;
                        btnSendFile.Enabled = true;
                        btnAdd.Enabled = false;
                        btnSearch.Enabled = true;
                    }

                }
                else
                {
                    txtMyName.Enabled = true;
                    txtPassword.Enabled = true;
                    txtPeerName.Enabled = true;

                    btnStart.Enabled = true;
                    btnStop.Enabled = false;
                    btnConnect.Enabled = false;
                    btnSend.Enabled = false;
                    btnSendFile.Enabled = false;
                    btnAdd.Enabled = false;
                    btnSearch.Enabled = false;
                }


            }
        }

        public void AddToList(string RefText)
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new Action<string>(AddToList), RefText);
            }
            else
            {
                List1.Items.Add(RefText);
                List1.SelectedIndex = List1.Items.Count - 1;
                List1.SelectedIndex = -1;
            }
        }

        private void Text6_TextChanged(System.Object eventSender, System.EventArgs eventArgs)
        {
            EnableButtons();
        }


        private void Timer1_Tick(System.Object eventSender, System.EventArgs eventArgs)
        {
            // not important code.. just to flash buttons
            // to lead you the way...
            if (!string.IsNullOrEmpty(Text6.Text))
            {
                if (btnConnect.BackColor == Color.Transparent)
                {
                    btnConnect.BackColor = Color.DarkGray;
                }
                else
                {
                    btnConnect.BackColor = Color.Transparent;
                }
            }
            else
            {
                if (btnSearch.BackColor == Color.Transparent)
                {
                    btnSearch.BackColor = Color.DarkGray;
                }
                else
                {
                    btnSearch.BackColor = Color.Transparent;
                }
            }

            TickCounter = TickCounter + 1;
            if (TickCounter == 6)
                Timer1.Enabled = false;

        }

        public bool P2P_ForwardOpenEventImpl(IntPtr Handle, int forwardid, int forwardtype, string localaddress, int localport, string remoteaddress, int remoteport)
        {
            AddToList("CHANNEL on port " + P2P.Forward_GetLocalPort(P2P.GetForward(Handle, forwardid)) + " started");
            return true;
            // allow
        }

        public void P2P_ForwardCloseEventImpl(IntPtr Handle, int forwardid, int errorno)
        {
            StringBuilder buffer = new StringBuilder(512);
            P2P.ErrorText(Handle, errorno, buffer);
            AddToList("CHANNEL on port " + P2P.Forward_GetLocalPort(P2P.GetForward(Handle, forwardid)) + " stopped " + buffer.ToString());
        }

        public void P2P_LinkDoneEventImpl(IntPtr Handle, string address, int port, int errorno)
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new P2P.LinkDoneEvent(P2P_LinkDoneEventImpl), new object[] {
				Handle,
				address,
				port,
				errorno
			});
            }
            else
            {
                EnableButtons();
                if (errorno == 0)
                {
                    String conntype;
                    switch ((P2P.ConnectionTypes)P2P.GetConnectionType(Handle))
                    {
                        case P2P.ConnectionTypes.ConnectionTCPDirect:
                            conntype = "TCP";
                            break;

                        case P2P.ConnectionTypes.ConnectionTCPRelayed:
                            conntype = "TCP Relayed";
                            break;

                        case P2P.ConnectionTypes.ConnectionUDPDirect:
                            conntype = "UDP";
                            break;

                        default:
                            conntype = "UNKNOWN";
                            break;
                    }
                    AddToList("CONNECTED " + conntype + " to " + address + " (on port " + port + ")");
                    Text6.Text = "";
                }
                else
                {
                    StringBuilder buffer = new StringBuilder(512);
                    P2P.ErrorText(p2phandle, errorno, buffer);
                    AddToList("CONNECT FAILED, " + buffer.ToString());
                }
            }
        }

        public void P2P_StoppedEventImpl(IntPtr Handle, int errorno)
        {
            EnableButtons();
            AddToList("DISCONNECTED");
        }

        public void P2P_TextReceivedEventImpl(IntPtr Handle, int chanid, string text)
        {
            AddToList("GOT MESSAGE:");
            AddToList(text);
        }

        public void P2P_SearchDoneEventImpl(IntPtr Handle, string peerName, string peerXML, string customData, int errorno)
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new P2P.SearchDoneEvent(P2P_SearchDoneEventImpl), new object[] {
				Handle,
                peerName,
				peerXML,
                customData,
				errorno
			});
            }
            else
            {
                if (errorno == 0)
                {
                    AddToList("SEARCH done, remote IP set.");
                    Text6.Text = peerXML;

                    // flash Connect button
                    TickCounter = 0;
                    Timer1.Enabled = true;
                }
                else
                {
                    StringBuilder buffer = new StringBuilder(512);
                    P2P.ErrorText(Handle, errorno, buffer);
                    AddToList("SEARCH " + buffer.ToString());
                }
            }
        }

        public void P2P_StateChangeEventImpl(IntPtr Handle, int state)
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new P2P.StateChangeEvent(P2P_StateChangeEventImpl), new object[] {
				Handle,
				state
			});
            }
            else
            {
                EnableButtons();
                StringBuilder buffer = new StringBuilder(512);
                P2P.StateText(Handle, state, buffer);
                AddToList("STATE " + buffer.ToString());
            }
        }

        public bool P2P_UserConnectedEventImpl(IntPtr Handle, int forwardid, int chanid, IntPtr userhandle)
        {
            StringBuilder buffer = new StringBuilder(512);
            P2P.Forward_GetRemoteAddress(P2P.GetForward(Handle, forwardid), buffer);
            AddToList("USER CONNECTED " + buffer.ToString());
            return true;
            // allow
        }

        public void P2P_UserDisconnectedEventImpl(IntPtr Handle, int forwardid, int chanid, IntPtr userhandle, int errorno)
        {
            StringBuilder buffer = new StringBuilder(512);
            P2P.Forward_GetRemoteAddress(P2P.GetForward(Handle, forwardid), buffer);
            AddToList("USER DISCONNECTED " + buffer.ToString());
        }

        public void P2P_PingEventImpl(IntPtr Handle)
        {
            AddToList("PING..");
        }

        public void btnStart_Click(System.Object sender, System.EventArgs ev)
        {
            P2P.SetMyName(p2phandle, txtMyName.Text);
            P2P.SetPeerName(p2phandle, txtPeerName.Text);
            P2P.SetPassword(p2phandle, 1, txtPassword.Text);

            //P2P.SetVPNadapter(p2phandle, "WeOnlyDo");

            int e = 0;
            P2P.SetTCPPort(p2phandle, 0);
            P2P.SetUDPPort(p2phandle, 0);
            e = P2P.Start(p2phandle, (int)P2P.Protocols.ProtoBoth);
            if ((e != 0))
            {
                StringBuilder buffer = new StringBuilder(512);
                P2P.ErrorText(Handle, e, buffer);
                MessageBox.Show(buffer.ToString());
                EnableButtons();
            }
            else
            {
                AddToList(("STARTED listening on TCP port " + P2P.GetTCPPort(p2phandle) + "  and UDP port " + P2P.GetUDPPort(p2phandle)));
                EnableButtons();
                Timer1.Enabled = true;
            }

        }

        private void btnStop_Click_1(System.Object sender, System.EventArgs e)
        {
            try
            {
                P2P.Stop(p2phandle);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        public delegate void Timer2_TickDelegate(System.Object sender, System.EventArgs e);
        public void Timer2_Tick(System.Object sender, System.EventArgs e)
        {
            return;
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new Timer2_TickDelegate(Timer2_Tick), new object[] {
				sender,
				e
			});
            }
            else
            {
                do
                {
                    int n = 0;
                    Random rnd = new Random();
                    n = rnd.Next(1, 5);

                    if (btnStart.Enabled & n == 1)
                    {
                        btnStart_Click(sender, e);
                        return;
                    }
                    if (btnStop.Enabled & n == 2)
                    {
                        btnStop_Click_1(sender, e);
                        return;
                    }
                    if (btnSearch.Enabled & n == 3)
                    {
                        btnSearch_Click(sender, e);
                        return;
                    }
                    if (btnConnect.Enabled & n == 4)
                    {
                        btnConnect_Click(sender, e);
                        return;
                    }
                } while (true);
            }
        }

        private void btnSendFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog file = new OpenFileDialog();
            if (file.ShowDialog() == DialogResult.OK)
                P2P.SendFile(p2phandle, 100, file.FileName, "newfilename", false);
        }
        private bool p2p_FileReceiveEventImpl(IntPtr Handle, int chanid, string filename, Int64 size)
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new P2P.FileReceiveEvent(p2p_FileReceiveEventImpl), new object[] {
				Handle,
				chanid,
				filename, 
                size
			});
            }
            else
            {
                AddToList("FILE RECEIVE: " + filename + "(" + size.ToString() + " bytes)");
            }
            return true;
        }
        private void p2p_FileProgressEventImpl(IntPtr Handle, int chanid, Int64 position, Int64 size)
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new P2P.FileProgressEvent(p2p_FileProgressEventImpl), new object[] {
				Handle,
				chanid,
				position, 
                size
			});
            }
            else
            {
                AddToList("FILE PROGRESS " + position.ToString());
            }
        }
        private void p2p_FileDoneEventImpl(IntPtr Handle, int chanid, int errorno)
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new P2P.FileDoneEvent(p2p_FileDoneEventImpl), new object[] {
				Handle,
				chanid,
				errorno, 
			});
            }
            else
            {
                StringBuilder buffer = new StringBuilder(512);
                P2P.ErrorText(p2phandle, errorno, buffer);
                AddToList("FILE TRANSFER COMPLETED (" + buffer.ToString() + ")");
            }
        }
    }
}
