﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using devP2P;

namespace SimpleVPN
{
    public partial class Form1 : Form
    {

        // globals - to keep garbage collector from releasing them        
        VPN.StateChangeEvent MyStateChangeEvent;
        VPN.PeerConnectingEvent MyPeerConnectingEvent;
        VPN.PeerConnectedEvent MyPeerConnectedEvent;
        VPN.PeerDisconnectedEvent MyPeerDisconnectedEvent;

        int CurrentInterfaceIndex = 0;

        public Form1()
        {
            InitializeComponent();

            MyStateChangeEvent = new VPN.StateChangeEvent(VPN_StateChangeEventImpl);
            MyPeerConnectingEvent = new VPN.PeerConnectingEvent(VPN_PeerConnectingEventImpl);
            MyPeerConnectedEvent = new VPN.PeerConnectedEvent(VPN_PeerConnectedEventImpl);
            MyPeerDisconnectedEvent = new VPN.PeerDisconnectedEvent(VPN_PeerDisconnectedEventImpl);

        }
        public IntPtr vpnhandle;

        private void Form1_FormClosed(object sender, System.Windows.Forms.FormClosedEventArgs e)
        {
            if (vpnhandle != IntPtr.Zero)
                VPN.Destroy(vpnhandle);

            Application.Exit();
        }


        private void Form1_Load(System.Object sender, System.EventArgs e)
        {
            try
            {
                vpnhandle = VPN.Create();
            }
            catch (System.Exception ex)
            {
                MessageBox.Show("Please check your dll path in cs source of this project!\r\n" + ex.Message, "Failed to load dll");
                Application.Exit();
                return;            	
            }

            VPN.RegisterStateChangeEvent(vpnhandle, MyStateChangeEvent);
            VPN.RegisterPeerConnectingEvent(vpnhandle, MyPeerConnectingEvent);
            VPN.RegisterPeerConnectedEvent(vpnhandle, MyPeerConnectedEvent);
            VPN.RegisterPeerDisconnected(vpnhandle, MyPeerDisconnectedEvent);

            StringBuilder buffer = new StringBuilder(512);
            VPN.GetVersion(vpnhandle, buffer);
            AddToList("devP2P VERSION " + buffer.ToString());

            Random rnd = new Random();
            int i;
            i = rnd.Next(100);
            txtMyName.Text = "myhost_" + i.ToString();
            txtSearchPeers.Text = "mypeers_" + i.ToString() + "*";
            CurrentInterfaceIndex = 0;

            EnableButtons();
            ShowFrameData();

        }

        private delegate void EnableButtonsDelegate();

        private void EnableButtons()
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new EnableButtonsDelegate(EnableButtons));

            }
            else
            {

                if (VPN.GetState(vpnhandle) > 0)
                {
                    btnStart.Enabled = false;
                    btnStop.Enabled = true;
                    btnSearch.Enabled = true;

                }
                else
                {
                    btnStart.Enabled = true;
                    btnStop.Enabled = false;
                    btnSearch.Enabled = false;
                }

            }
        }

        public void AddToList(string RefText)
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new Action<string>(AddToList), RefText);
            }
            else
            {
                List1.Items.Add(RefText);
                List1.SelectedIndex = List1.Items.Count - 1;
                List1.SelectedIndex = -1;
            }
        }


        public void VPN_StateChangeEventImpl(IntPtr Handle, int state)
        {
            if (List1.InvokeRequired)
            {
                List1.BeginInvoke(new VPN.StateChangeEvent(VPN_StateChangeEventImpl), new object[] {
				Handle,
				state
			});
            }
            else
            {
                EnableButtons();
                StringBuilder buffer = new StringBuilder(512);
                VPN.StateText(Handle, state, buffer);
                AddToList("STATE " + buffer.ToString());
            }
        }
        public bool VPN_PeerConnectingEventImpl(IntPtr Handle, IntPtr p2pHandle, string peerName, string peerAddrs)
        {
            AddToList("PEERCONNECTING " + peerName );
            return true;
        }
        public void VPN_PeerConnectedEventImpl(IntPtr Handle, IntPtr p2pHandle, string address, int port)
        {
            StringBuilder buffer = new StringBuilder(512);
            P2P.GetPeerAdapterIP(p2pHandle, buffer);
            AddToList("PEERCONNECTED " + buffer.ToString());
        }
        public void VPN_PeerDisconnectedEventImpl(IntPtr Handle, IntPtr p2pHandle, int ErrorNo)
        {
            StringBuilder buffer = new StringBuilder(512);
            P2P.GetMyName(p2pHandle, buffer);
            AddToList("PEERDISCONNECTED " + buffer.ToString());

        }


        public void ShowFrameData()
        {
            StringBuilder buffer = new StringBuilder(512);

            if (CurrentInterfaceIndex < VPNAdapter.Count())
            {

                VPNAdapter.GetName(CurrentInterfaceIndex, buffer);
                txtAdapterName.Text = buffer.ToString();

                VPNAdapter.GetDriverName(CurrentInterfaceIndex, buffer);
                txtAdapterDriverName.Text = buffer.ToString();

                VPNAdapter.GetLocalIP(CurrentInterfaceIndex, buffer);
                txtAdapterLocalIP.Text = buffer.ToString();

                VPNAdapter.GetLocalNetmask(CurrentInterfaceIndex, buffer);
                txtAdapterLocalNetmask.Text = buffer.ToString();

                VPNAdapter.GetLocalMAC(CurrentInterfaceIndex, buffer);
                txtAdapterLocalMAC.Text = buffer.ToString();


            }

            Label7.Text = (CurrentInterfaceIndex + 1).ToString() + "/" + VPNAdapter.Count();
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            VPN.SetMyName(vpnhandle, txtMyName.Text);
            if (CurrentInterfaceIndex < VPNAdapter.Count())
                VPN.SetAdapter(vpnhandle, CurrentInterfaceIndex);

            int ret = VPN.Start(vpnhandle, 2);
            if (ret != 0)
            {
                StringBuilder buffer = new StringBuilder(512);
                VPN.ErrorText(vpnhandle, ret, buffer);
                AddToList("ERROR " + buffer.ToString());
            }
            else
                AddToList("Done.. Now Search for peers..");
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            VPN.Stop(vpnhandle);
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            VPN.Search(vpnhandle, txtSearchPeers.Text, "", true);
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            if (CurrentInterfaceIndex > 0)
                CurrentInterfaceIndex = CurrentInterfaceIndex - 1;

            ShowFrameData();
        }

        private void Button2_Click(object sender, EventArgs e)
        {
            if (CurrentInterfaceIndex < VPNAdapter.Count() - 1)
                CurrentInterfaceIndex = CurrentInterfaceIndex + 1;

            ShowFrameData();
        }

        private void btnSetIP_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(
                     "Are you sure you want to set interface " + txtAdapterName.Text + " to:\r\n" +
                     "IP address " + txtAdapterLocalIP.Text + "\r\nNetmask " + txtAdapterLocalNetmask.Text + ".\r\n\r\nNOTE: external program will be called to set interface address (netsh)\r\n\r\nAdministrator privileges are required to succeed!",
                     "Change IP address",
                     MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                //String netsh = "netsh interface ip set address name=\"" + txtAdapterName.Text + "\" static " + txtAdapterLocalIP.Text + " " + txtAdapterLocalNetmask.Text;
                this.Cursor = Cursors.WaitCursor;
                VPNAdapter.SetIP(CurrentInterfaceIndex, txtAdapterLocalIP.Text, txtAdapterLocalNetmask.Text);
                //Shell(netsh, AppWinStyle.NormalFocus, True, 15);
                this.Cursor = Cursors.Default;
            }
        }
    }
}
