#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef _WIN32
#include <windows.h>
#include <conio.h>
#pragma warning(disable: 4996)
#ifdef _WIN64
#pragma comment(lib, "../../../bin/Windows_x64_lib/devP2P.lib")
#else
#pragma comment(lib, "../../../bin/Windows_x86_lib/devP2P.lib")
#endif
#pragma comment(lib, "wsock32.lib")
#pragma comment(lib, "winmm.lib")
#pragma comment(lib, "iphlpapi.lib")

#if (defined (_MSC_VER) && defined (_DEBUG))
//HACK: allow the use of mixed debug/non-debug code
extern "C" {
	_CRTIMP void __cdecl _invalid_parameter_noinfo(void) {  }
}
#endif

#else
#define getch getchar
int Sleep(unsigned long milisec);
#define _stricmp strcasecmp
#endif
#include "../../../inc/p2p.h"

void vpn_StateChange(devP2Plib::CVPN *vpn, devP2Plib::States state)
{
	printf("State changed to %s\r\n", vpn->StateText(state));
}

bool vpn_PeerConnecting(devP2Plib::CVPN *vpn, devP2Plib::CP2P *p2p, char *peerName, char *peerAddrs)
{
	printf("PeerConnecting %s\r\n", peerName);
	return true;
}

void vpn_PeerConnected(devP2Plib::CVPN *vpn, devP2Plib::CP2P *p2p, char *address, int port)
{
	struct in_addr ina;
	ina.s_addr = p2p->PeerAdapterIP;
	printf("PeerConnected %s, IP %s\r\n", p2p->PeerName, inet_ntoa(ina));
}

void vpn_PeerDisconnected(devP2Plib::CVPN *vpn, devP2Plib::CP2P *p2p, devP2Plib::Errors error)
{
	struct in_addr ina;
	ina.s_addr = p2p->PeerAdapterIP;
	printf("PeerDisconnected %s, IP %s (%s)\r\n", p2p->PeerName, inet_ntoa(ina), vpn->ErrorText(error));
}


void Usage(void)
{
	printf("SimpleVPN version 0.9, WeOnlyDo Software (2014)\r\n");
	printf("--\r\n");
	printf("Usage: SimpleVPN MyName PeerNames Adapter [options]\r\n");
	printf("  MyName, PeerName and Adapter name must be provided!\r\n");
	printf("  Options can be:\r\n");
	printf("    -password <desired.password>\r\n");
	printf("    -mediatorip <ip.address.of.mediator>\r\n");
	printf("    -mediatorport <port.of.mediator>\r\n");
	printf("    -udp use only udp\r\n");
	printf("    -tcp use only tcp\r\n");
	printf("    -relaylicense <your.relay.license>\r\n");
#if !defined(__CYGWIN__)
	printf("    -listadapters lists all VPN adapters\r\n");
	printf("    -setip <ip.ad.re.ss> set IP for the adapter\r\n");
	printf("    -setnemask <ne.tm.as.k> set netmask for the adapter\r\n");
#endif
	printf("\r\n");
}


int main(int argc, char **argv)
{
#ifdef _WIN32
	WSADATA m_wsadata;
	WSAStartup(MAKEWORD(2, 0), &m_wsadata);
#endif

	devP2Plib::libInit();
	devP2Plib::upnpInit();
	Sleep(200);

	devP2Plib::CVPN *vpnhost;

	vpnhost = devP2Plib::CVPN::Create();
	vpnhost->Events.PeerConnected = vpn_PeerConnected;
	vpnhost->Events.PeerConnecting = vpn_PeerConnecting;
	vpnhost->Events.PeerDisconnected = vpn_PeerDisconnected;
	vpnhost->Events.StateChange = vpn_StateChange;

	int proto = 0;
	char password[1024] = {0};
	char ip[128];
	char peernames[1024] = {0};
	memset(ip, 0, sizeof(ip));
	char netmask[128];
	memset(netmask, 0, sizeof(netmask));

#if !defined(__CYGWIN__)
	int VPNcount = devP2Plib::VPNAdapterCount();
#endif
	devP2Plib::CVPNInterface *vpn = NULL;

	int i=1;
	while (i<argc)
	{
		if (*argv[i] == '-' || *argv[i]=='/')
		{
			char *a = argv[i];
			while (*a && (*a == '-' || *a == '/')) a++;

			if (!_stricmp(a, "mediatorip"))
			{
				i++;
				if (i < argc)
				{
					vpnhost->MediatorAddress = argv[i];
					i++;
				}
			}
			else
			if (!_stricmp(a, "password"))
			{
				i++;
				if (i < argc)
				{
					strcpy(password, argv[i]);
					i++;
				}
			}
			else
			if (!_stricmp(a, "relaylicense"))
			{
				i++;
				if (i < argc)
				{
					vpnhost->RelayLicense = argv[i];
					i++;
				}
			}
			else
			if (!_stricmp(a, "mediatorport"))
			{
				i++;
				if (i < argc)
				{
					vpnhost->MediatorPort = atoi(argv[i]);
					i++;
				}
			}
			else
			if (!_stricmp(a, "udp"))
			{
				i++;
				proto |= devP2Plib::ProtoUDP;
			}
			else
			if (!_stricmp(a, "tcp"))
			{
				i++;
				proto |= devP2Plib::ProtoTCP;
			}

			else
#if !defined(__CYGWIN__)
			if (!_stricmp(a, "listadapters"))
			{
				i++;
				printf("---Local adapters available---\r\n");
				for (int j=0;j<VPNcount;j++)
				{
					devP2Plib::CVPNInterface *vpn = devP2Plib::VPNAdapter(j);
					struct in_addr ina1, ina2;
					ina1.s_addr = vpn->LocalIP;
					ina2.s_addr = vpn->LocalNetmask;
					printf("ID %d:  Name: %s  ", j, vpn->Name);
					printf("IP: %s  ", inet_ntoa(ina1));
					printf("Netmask: %s  ", inet_ntoa(ina2));
					char buf[19];
					memset(buf, 0, sizeof(buf));
					devP2Plib::MacToText(vpn->LocalMAC, buf);
					printf("Mac: %s\r\n", buf);
				}
				printf("---End of listing---\r\n");
			}
			else
			if (!_stricmp(a, "setip"))
			{
				i++;
				strcpy(ip, argv[i]);
				i++;
			}
			else
				if (!_stricmp(a, "setnetmask"))
				{
					i++;
					strcpy(netmask, argv[i]);
					i++;
				}
#endif // ifdef cygwin
			else
				i++;
		}
		else
		{
			if (!vpnhost->MyName)
				vpnhost->MyName = argv[i];
			else
			if (!*peernames)
				strcpy(peernames, argv[i]);
			else
			{
				devP2Plib::CVPNInterface *vpn1 = NULL;
#if defined(__APPLE__) || defined(__FreeBSD__)
				vpn1 = devP2Plib::VPNAdapter(0);
				if (vpn1)
				{
					strcpy(vpn1->Name, argv[i]);
					vpn = vpn1;
				}
#else
				for (int j=0;j<VPNcount;j++)
				{
					vpn1 = devP2Plib::VPNAdapter(j);	
					if (vpn1 && !strcmp(vpn1->Name, argv[i]))
					{
						vpn = vpn1;
						break;
					}
				}
#endif
			}
			i++;
		}
	}


	if (!vpnhost->MyName || !*peernames)
	{
		Usage();
		exit(-1);
	}

	if (!vpn)
	{
		printf("ERROR: invalid adapter selected\r\n");
		exit(-1);
	}

	if (vpnhost->SetAdapter(vpn) == devP2Plib::ErrorNone)
	{
		if (*ip)
			vpn->SetIP(ip, netmask);

		struct in_addr ina1;
		ina1.s_addr = vpn->LocalIP;
		printf("Using VPN adapter %s <%s>\r\n", vpn->Name, inet_ntoa(ina1));
	}
	else
	{
		printf("ERROR: %s\r\n", vpnhost->ErrorText(devP2Plib::ErrorAdapterInvalid));
		exit(-1);
	}

	if (!proto)
		proto = devP2Plib::ProtoBoth;

	int success = vpnhost->Start((devP2Plib::Protocols)proto);

	if (success >= 0)
		vpnhost->Search(peernames, password, true);

	if (success >= 0)
	{
		int key;
		bool cont = true;
		do 
		{
			key = getch();
			switch (key)
			{
			case 'x':
				cont = false;
				break;
			}
		} while (cont);
	}

	if (success < 0)
		printf("Error: %s\r\n", vpnhost->ErrorText((devP2Plib::Errors)success));


	vpnhost->Stop();
	vpnhost->Destroy();
	devP2Plib::libFree();

#ifdef _WIN32
	WSACleanup();
#endif
	return 0;
}
