Option Explicit On

Imports System.Runtime.InteropServices
Imports System.Text
Imports System.Timers

Friend Class Form1
    Inherits System.Windows.Forms.Form

    Public vpnhandle As IntPtr
    Public CurrentInterfaceIndex As Integer

    'public static, so they are not destroyed by GC
    Dim MyStateChangeEvent As VPN_StateChangeEvent
    Dim MyPeerConnectingEvent As VPN_PeerConnectingEvent
    Dim MyPeerConnectedEvent As VPN_PeerConnectedEvent
    Dim MyPeerDisconnectedEvent As VPN_PeerDisconnectedEvent

    Private Sub Form1_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        If vpnhandle <> 0 Then
            VPN_Destroy(vpnhandle)
        End If
    End Sub

    Private Sub btnSearch_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles btnSearch.Click
        VPN_Search(vpnhandle, txtSearchPeers.Text, "", True)
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Try
            vpnhandle = VPN_Create()
        Catch ex As Exception
            MsgBox("Please check for devP2P.DLL path in StartModule.vb" & vbCrLf & ex.Message, MsgBoxStyle.OkOnly, "Failed to load devP2P.dll")
            Application.Exit()
            Exit Sub
        End Try

        MyStateChangeEvent = New VPN_StateChangeEvent(AddressOf VPN_StateChangeEventImpl)
        MyPeerConnectingEvent = New VPN_PeerConnectingEvent(AddressOf VPN_PeerConnectingEventImpl)
        MyPeerConnectedEvent = New VPN_PeerConnectedEvent(AddressOf VPN_PeerConnectedEventImpl)
        MyPeerDisconnectedEvent = New VPN_PeerDisconnectedEvent(AddressOf VPN_PeerDisconnectedEventImpl)


        VPN_RegisterStateChangeEvent(vpnhandle, MyStateChangeEvent)
        VPN_RegisterPeerConnectingEvent(vpnhandle, MyPeerConnectingEvent)
        VPN_RegisterPeerConnectedEvent(vpnhandle, MyPeerConnectedEvent)
        VPN_RegisterPeerDisconnectedEvent(vpnhandle, MyPeerDisconnectedEvent)


        Dim buffer As New StringBuilder(512)
        VPN_GetVersion(vpnhandle, buffer)
        AddToList("devP2P VERSION " + buffer.ToString)


        Dim i As Integer
        i = CInt(Math.Ceiling(Rnd() * 100))
        txtMyName.Text = "myhost_" & i
        txtSearchPeers.Text = "mypeers_" & i & "*"


        CurrentInterfaceIndex = 0
        EnableButtons()
        ShowFrameData()


    End Sub

    Private Delegate Sub EnableButtonsDelegate()
    Private Sub EnableButtons()

        Dim state As Integer

        If List1.InvokeRequired Then
            List1.BeginInvoke(New EnableButtonsDelegate(AddressOf EnableButtons))
        Else

            state = VPN_GetState(vpnhandle)

            If state > 0 Then
                ' started
                btnStart.Enabled = False
                btnStop.Enabled = True
                btnSearch.Enabled = True

                Button1.Enabled = False
                Button1.Enabled = False
            Else
                btnStart.Enabled = True
                btnStop.Enabled = False
                btnSearch.Enabled = False

                Button1.Enabled = True
                Button1.Enabled = True

            End If
        End If
    End Sub

    Public Sub AddToList(ByVal RefText As String)
        If List1.InvokeRequired Then
            List1.BeginInvoke(New Action(Of String)(AddressOf AddToList), RefText)
        Else
            List1.Items.Add(RefText)
            List1.SelectedIndex = List1.Items.Count - 1
            List1.SelectedIndex = -1
        End If
    End Sub

    Public Sub VPN_StateChangeEventImpl(ByVal Handle As IntPtr, ByVal state As Integer)
        Dim buffer As New StringBuilder(512)
        VPN_StateText(vpnhandle, state, buffer)

        AddToList("STATE " & buffer.ToString)

        EnableButtons()

    End Sub
    Public Function VPN_PeerConnectingEventImpl(ByVal Handle As IntPtr, ByVal P2PHandle As IntPtr, ByVal peerName As String, ByVal peerAddrs As String) As Integer
        AddToList("PEERCONNECTING " & peerName)
        Return True
    End Function
    Public Sub VPN_PeerConnectedEventImpl(ByVal Handle As IntPtr, ByVal P2PHandle As IntPtr, ByVal address As String, ByVal port As Integer)

        Dim buffer As New StringBuilder(512)
        P2P_GetPeerAdapterIP(P2PHandle, buffer)

        AddToList("PEERCONNECTED " & buffer.ToString())
    End Sub
    Public Sub VPN_PeerDisconnectedEventImpl(ByVal Handle As IntPtr, ByVal P2PHandle As IntPtr, ByVal errorno As Integer)
        Dim buffer As New StringBuilder(512)
        P2P_GetMyName(P2PHandle, buffer)

        AddToList("PEERDISCONNECTED " & buffer.ToString())
    End Sub


    Public Sub btnStart_Click(ByVal sender As System.Object, ByVal ev As System.EventArgs) Handles btnStart.Click
        VPN_SetMyName(vpnhandle, txtMyName.Text)
        If CurrentInterfaceIndex < VPNAdapter_Count() Then
            VPN_SetAdapter(vpnhandle, CurrentInterfaceIndex)
        End If


        Dim err As Integer

        err = VPN_Start(vpnhandle, P2P_Protocols.ProtoBoth)
        If err <> 0 Then
            Dim buffer As New StringBuilder(512)
            VPN_ErrorText(vpnhandle, err, buffer)
            AddToList("ERROR: " & buffer.ToString)
        Else
            AddToList("Done.. Now Search for peers...")
        End If


    End Sub

    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click
        Try
            VPN_Stop(vpnhandle)
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    Private Sub btnSetIP_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSetIP.Click
        Dim result As Integer

        result = MsgBox("Are you sure you want to set interface " & txtAdapterName.Text & " to:" & vbCrLf & _
        "IP address " & txtAdapterLocalIP.Text & vbCrLf & "Netmask " & txtAdapterLocalNetmask.Text & "." & vbCrLf & vbCrLf & _
        "NOTE: external program will be called to set interface address (netsh)" & vbCrLf & vbCrLf & _
        "Administrator privileges are required to succeed!", vbYesNo, "Change IP address")
        If result = 6 Then
            Dim netsh As String
            netsh = "netsh interface ip set address name=""" & txtAdapterName.Text & """ static " & txtAdapterLocalIP.Text & " " & txtAdapterLocalNetmask.Text
            Me.Cursor = Cursors.WaitCursor
            Shell(netsh, AppWinStyle.NormalFocus, True, 15)
            Me.Cursor = Cursors.Default
        End If
    End Sub

    Private Sub ShowFrameData()
        Dim buffer As New StringBuilder(512)

        If CurrentInterfaceIndex < VPNAdapter_Count() Then

            VPNAdapter_GetName(CurrentInterfaceIndex, buffer)
            txtAdapterName.Text = buffer.ToString

            VPNAdapter_GetDriverName(CurrentInterfaceIndex, buffer)
            txtAdapterDriverName.Text = buffer.ToString

            VPNAdapter_GetLocalIP(CurrentInterfaceIndex, buffer)
            txtAdapterLocalIP.Text = buffer.ToString

            VPNAdapter_GetLocalNetmask(CurrentInterfaceIndex, buffer)
            txtAdapterLocalNetmask.Text = buffer.ToString

            VPNAdapter_GetLocalMAC(CurrentInterfaceIndex, buffer)
            txtAdapterLocalMAC.Text = buffer.ToString


        End If

        Label7.Text = (CurrentInterfaceIndex + 1).ToString & "/" & VPNAdapter_Count()
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        If CurrentInterfaceIndex > 0 Then
            CurrentInterfaceIndex = CurrentInterfaceIndex - 1
        End If

        ShowFrameData()
    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        If CurrentInterfaceIndex < VPNAdapter_Count() - 1 Then
            CurrentInterfaceIndex = CurrentInterfaceIndex + 1
        End If

        ShowFrameData()
    End Sub
End Class